@@ -20,7 +20,7 @@ class Logger:
 	# index of output
 	id      = 0
 
-	def __init__(self, path: str= 'log', filename: str= 'access', extension: str= '.log', enableLog: bool= True, enableConsole: bool= True, line: bool= True, color: bool= True):
+	def __init__(self, path: str = 'log', filename: str = 'access', extension: str = 'log', enableLog: bool = True, enableConsole: bool = True, line: bool = True, charInLine: int= 55, lineCharStart: str = '>', lineCharEnd: str = '<', color: bool = True):
 		"""
 
 		:param path:
@@ -29,20 +29,25 @@ class Logger:
 		:param enableLog:
 		:param enableConsole:
 		:param line:
+		:param charInLine:
+		:param lineCharStart:
+		:param lineCharEnd:
 		:param color:
 		"""
 		# datetime format
 		# ex: 2020-05-18
-		self.__formatFileName   = '%Y-%m-%d'
+		self.__formatFileName   	= '%Y-%m-%d'
 		# full datetime format
 		# ex: 2022-05-12 21:40:20.345
-		self.__dateTimeFormat   = f'{self.__formatFileName} %H:%M:%S.%f'
+		self.__dateTimeFormat   	= f'{self.__formatFileName} %H:%M:%S'
 		# set color
-		self.__color            = color
-		self.__line             = line
+		self.__color            	= color
+		self.__line             	= line
+		self.__lineCharStart        = f'{lineCharStart}' * charInLine
+		self.__lineCharEnd          = f'{lineCharEnd}' * charInLine
 		# config
-		self.__enableConsole    = enableConsole
-		self.__enableLog        = enableLog
+		self.__enableConsole    	= enableConsole
+		self.__enableLog        	= enableLog
 		# redis
 		self.__enableRedis			= False
 		self.__enableRedisError		= False
@@ -52,20 +57,20 @@ class Logger:
 		self.__enableRedisTrack		= False
 		self.__enableRedisWarning	= False
 		#
-		self.__extension        = extension
+		self.__extension        	= f'.{extension}'
 		# path + /
-		self.__path             = path
+		self.__path            		= path
 		# compute
-		self.__filename         = os.path.join(self.__path, f'{filename}{self.__extension}')
+		self.__filename         	= os.path.join(self.__path, f'{filename}{self.__extension}')
 		# static datetime
-		self.__datetime         = datetime.now().strftime(self.__dateTimeFormat)
+		self.__datetime         	= datetime.now().strftime(self.__dateTimeFormat)
 		# series
-		self.__keySeries        = ''
+		self.__keySeries        	= ''
 		# session as uuid or md5
-		self.__keySession       = ''
+		self.__keySession       	= ''
 		# pub/sub
 		# self.__fcm				= FCMLib()
-		self.__redis			= RedisLib()
+		self.__redis				= RedisLib()
 		# load
 
 	def __createNewBackupFile(self) -> None:
@@ -91,7 +96,7 @@ class Logger:
 		:return:
 		"""
 		# generate a filename
-		return f'{self.__path}{(datetime.now() - timedelta(1)).strftime(self.__formatFileName)}{self.__extension}'
+		return os.path.join(self.__path, f'{(datetime.now() - timedelta(1)).strftime(self.__formatFileName)}{self.__extension}')
 
 	def __getContentBody(self, typeName: str, title: str, content: Any, color: str) -> str:
 		"""
@@ -102,19 +107,15 @@ class Logger:
 		:param color:
 		:return:
 		"""
-		lineStart   = ''
-		lineEnd     = ''
-		cHead       = ''
 		cBody       = f'[{typeName}] '
 		cFoot       = ''
+		cHead       = ''
 
 		# add line to content
 		if bool(self.__line):
-			lineStart   = '>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>'
-			lineEnd     = '<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<'
 			# update
-			cHead       = lineStart
-			cFoot       = lineEnd
+			cHead       = self.__lineCharStart
+			cFoot       = self.__lineCharEnd
 
 		# add color, style
 		if bool(self.__color):
@@ -127,7 +128,6 @@ class Logger:
 			cBody       = f'{cBody}{title}'
 
 		# final content
-		# f"{cBody}{self.__getStr(content)} \n{datetime.now().strftime(self.__dateTimeFormat)}" \
 		return f'\n{cHead}\n{cBody}{self.__getStr(content)} \n{cFoot}\n\n' if bool(self.__line) else f'\n{cBody}\n'
 
 	def __getContentFile(self, fileName: str) -> str:
@@ -153,18 +153,19 @@ class Logger:
 		except Exception:
 			return ''
 
-	def __getContentHead(self, logId: int) -> str:
+	def __getContentHead(self, logId: int, idLabel: str= '') -> str:
 		"""
 
 		:param logId:
+		:param idLabel:
 		:return:
 		"""
 		# final data
 		if self.__keySeries:
-			return f'{self.__datetime} <id: {logId}> {self.__keySeries}\n'
+			return f'{self.__datetime} <{idLabel}{logId}> {self.__keySeries}\n'
 
 		else:
-			return f'{self.__datetime} <id: {logId}>\n'
+			return f'{self.__datetime} <{idLabel}{logId}>\n'
 
 	def __getStr(self, content: Any) -> str:
 		"""
@@ -181,13 +182,21 @@ class Logger:
 		else:
 			return ''
 
-	async def __msgRedis(self, channel: str= None) -> None:
+	def __isEnabledRedis(self, enabledLevel: bool) -> bool:
+		"""
+
+		:param enabledLevel:
+		:return:
+		"""
+		return self.__enableRedis and enabledLevel
+
+	def __msgRedis(self, channel: str= None) -> None:
 		"""
 
 		:param channel:
 		:return:
 		"""
-		self.__redis.messageGet()
+		self.__redis.messageGet(channel)
 
 	# def __pushFCM(self, title: str, body: str) -> None:
 	# 	"""
@@ -198,23 +207,22 @@ class Logger:
 	# 	"""
 	# 	pass
 
-	async def __pushRedis(self, title: str, body: str, channel: str= None) -> None:
+	def __pushRedis(self, isEnabled: bool, title: str, content: str, channel: str = None) -> None:
 		"""
 
+		:param isEnabled:
 		:param title:
-		:param body:
+		:param content:
 		:param channel:
 		:return:
 		"""
-		#
-		notification	= self.__redis.messagePush(
-			title		= title
-			, body		= body
-			, channel	= channel
-		)
-
-		#
-		print(notification)
+		if self.__isEnabledRedis(enabledLevel= isEnabled):
+			#
+			self.__redis.messagePush(
+				title		= title
+				, body		= content
+				, channel	= channel
+			)
 
 	def __setNewId(self, id: int) -> None:
 		"""
@@ -250,23 +258,23 @@ class Logger:
 		# do filter
 		if Logger.id not in Logger.hide:
 			# update content
-			contentBody = self.__getContentBody(typeName= typeName, title= title, content= self.__getStr(content= content), color= color)
-			contentHead = self.__getContentHead(logId= Logger.id)
+			contentBody = self.__getContentBody(typeName= typeName.upper(), title= title, content= self.__getStr(content= content), color= color)
+			contentHead = self.__getContentHead(logId= Logger.id, idLabel= 'id: ')
 
 			# enable file log
 			if bool(self.__enableLog):
 				# write to file
-				self.__writeFile(content= f'{contentHead}{contentBody}')
+				self.__writeFile(content= f'{contentHead}\n{contentBody}')
 
 				# verify first
 				if self.__keySession:
 					# output content to specific file via session's
-					self.__writeSessionFile(content= f'{contentHead}{contentBody}')
+					self.__writeSessionFile(content= f'{contentHead}\n{contentBody}')
 
 			# enable console
 			if bool(self.__enableConsole):
 				# console
-				print(f'{contentHead}{contentBody}')
+				print(f'{contentHead}\n{contentBody}')
 
 	def __writeFile(self, content: str) -> None:
 		"""
@@ -318,12 +326,13 @@ class Logger:
 		if numbers:
 			Logger.hide     = numbers
 
-	def error(self, title: str = '', content: Any = None, id: int = None) -> None:
+	def error(self, title: str = '', content: Any = None, id: int = None, channel: str = None) -> None:
 		"""
 
 		:param title:
 		:param content:
 		:param id:
+		:param channel:
 		:return:
 		"""
 		self.__write(
@@ -334,12 +343,21 @@ class Logger:
 			, logId     = id
 		)
 
-	def fail(self, title: str = '', content: Any= None, id: int = None) -> None:
+		#
+		self.__pushRedis(
+			isEnabled	= self.__redis.enableError
+			, title		= title
+			, content	= content
+			, channel	= channel
+		)
+
+	def fail(self, title: str = '', content: Any = None, id: int = None, channel: str = None) -> None:
 		"""
 
 		:param title:
 		:param content:
 		:param id:
+		:param channel:
 		:return:
 		"""
 		self.__write(
@@ -351,21 +369,23 @@ class Logger:
 		)
 
 		#
-		if self.__enableRedis and self.__redis.enableFail:
-			#
-			self.__redis.messagePush(
-				title	= title
-				, body	= content
-			)
+		self.__pushRedis(
+			isEnabled	= self.__redis.enableFail
+			, title		= title
+			, content	= content
+			, channel	= channel
+		)
 
-	def info(self, title: str = '', content: Any = None, id: int = None) -> None:
+	def info(self, title: str = '', content: Any = None, id: int = None, channel: str = None) -> None:
 		"""
 
 		:param title:
 		:param content:
 		:param id:
+		:param channel:
 		:return:
 		"""
+
 		self.__write(
 			typeName    = 'INFO'
 			, title     = title
@@ -375,12 +395,12 @@ class Logger:
 		)
 
 		#
-		if self.__enableRedis and self.__redis.enableInfo:
-			#
-			self.__redis.messagePush(
-				title	= title
-				, body	= content
-			)
+		self.__pushRedis(
+			isEnabled	= self.__redis.enableInfo
+			, title		= title
+			, content	= content
+			, channel	= channel
+		)
 
 	# def setFCM(self, config: dict) -> None:
 	# 	"""
@@ -398,7 +418,7 @@ class Logger:
 		"""
 		self.__keySeries    = series
 
-	def setKeySession(self, sessionKey: str= None) -> None:
+	def setKeySession(self, sessionKey: str = None) -> None:
 		"""
 
 		:note: set session filename
@@ -414,7 +434,7 @@ class Logger:
 			# empty or none won't able this feature
 			self.__keySession       = sessionKey
 
-	def setRedis(self, enable: bool, host: str, port: int, db: int= 0, password: str= None, enableFail: bool= False, enableInfo: bool= False, enableTrack: bool= False, enableSuccess: bool= False, enableWarning: bool= False, channel: str= None) -> None:
+	def setRedis(self, enable: bool, host: str, port: int, db: int = 0, password: str = None, enableError: bool = False, enableFail: bool = False, enableInfo: bool = False, enableTrack: bool = False, enableSuccess: bool = False, enableWarning: bool = False, channel: str = None) -> None:
 		"""
 
 		:param enable:
@@ -422,6 +442,12 @@ class Logger:
 		:param port:
 		:param db:
 		:param password:
+		:param enableError:
+		:param enableFail:
+		:param enableInfo:
+		:param enableTrack:
+		:param enableSuccess:
+		:param enableWarning:
 		:param channel:
 		:return:
 		"""
@@ -437,6 +463,7 @@ class Logger:
 		)
 
 		# enable level of pub/sub
+		self.__redis.enableError	= enableError
 		self.__redis.enableFail		= enableFail
 		self.__redis.enableInfo		= enableInfo
 		self.__redis.enableTrack	= enableTrack
@@ -446,11 +473,13 @@ class Logger:
 		# set channel name
 		self.__redis.channelSet(channel= channel)
 
-	def success(self, title: str = '', content: Any = None, id: int = None) -> None:
+	def success(self, title: str = '', content: Any = None, id: int = None, channel: str = None) -> None:
 		"""
+
 		:param title:
 		:param content:
 		:param id:
+		:param channel:
 		:return:
 		"""
 		self.__write(
@@ -462,19 +491,20 @@ class Logger:
 		)
 
 		#
-		if self.__enableRedis and self.__redis.enableSuccess:
-			#
-			self.__redis.messagePush(
-				title	= title
-				, body	= content
-			)
+		self.__pushRedis(
+			isEnabled	= self.__redis.enableSuccess
+			, title		= title
+			, content	= content
+			, channel	= channel
+		)
 
-	def track(self, title: str = '', content: Any = None, id: int = None) -> None:
+	def track(self, title: str = '', content: Any = None, id: int = None, channel: str = None) -> None:
 		"""
 
 		:param title:
 		:param content:
 		:param id:
+		:param channel:
 		:return:
 		"""
 		self.__write(
@@ -485,19 +515,20 @@ class Logger:
 		)
 
 		#
-		if self.__enableRedis and self.__redis.enableTrack:
-			#
-			self.__redis.messagePush(
-				title	= title
-				, body	= content
-			)
+		self.__pushRedis(
+			isEnabled	= self.__redis.enableTrack
+			, title		= title
+			, content	= content
+			, channel	= channel
+		)
 
-	def warning(self, title: str = '', content: Any = None, id: int = None) -> None:
+	def warning(self, title: str = '', content: Any = None, id: int = None, channel: str = None) -> None:
 		"""
 
 		:param title:
 		:param content:
 		:param id:
+		:param channel:
 		:return:
 		"""
 		self.__write(
@@ -509,12 +540,12 @@ class Logger:
 		)
 
 		#
-		if self.__enableRedis and self.__redis.enableWarning:
-			#
-			self.__redis.messagePush(
-				title	= title
-				, body	= content
-			)
+		self.__pushRedis(
+			isEnabled	= self.__redis.enableWarning
+			, title		= title
+			, content	= content
+			, channel	= channel
+		)
 
 	class __StyleModifier:
 		# Foreground
